@@ -31,7 +31,7 @@ from tacker.plugins.common import constants
 
 VIM_ATTRIBUTES = ('id', 'type', 'tenant_id', 'name', 'description',
                   'placement_attr', 'shared', 'is_default',
-                  'created_at', 'updated_at', 'status')
+                  'created_at', 'updated_at', 'status', 'extra')
 
 VIM_AUTH_ATTRIBUTES = ('auth_url', 'vim_project', 'password', 'auth_cred')
 
@@ -52,9 +52,20 @@ class NfvoPluginDb(nfvo.NFVOPluginBase, db_base.CommonDbMixin):
         res['auth_url'] = vim_auth_db[0].auth_url
         res['vim_project'] = vim_auth_db[0].vim_project
         res['auth_cred'] = vim_auth_db[0].auth_cred
-        res['auth_cred']['password'] = vim_auth_db[0].password
+        if vim_auth_db[0].password:
+            res['auth_cred']['password'] = vim_auth_db[0].password
+        # NOTE(Yao Qibin): Since oidc_token_url contains keyword `token`,
+        # its value will be masked.
+        # To prevent its value from being masked, temporarily change its name.
+        if "oidc_token_url" in res['auth_cred']:
+            res['auth_cred']['oidc_x_url'] = res['auth_cred'].pop(
+                'oidc_token_url')
         if mask_password:
             res['auth_cred'] = strutils.mask_dict_password(res['auth_cred'])
+        # Revert to oidc_token_url
+        if "oidc_x_url" in res['auth_cred']:
+            res['auth_cred']['oidc_token_url'] = res['auth_cred'].pop(
+                'oidc_x_url')
         return self._fields(res, fields)
 
     def _fields(self, resource, fields):
@@ -87,12 +98,13 @@ class NfvoPluginDb(nfvo.NFVOPluginBase, db_base.CommonDbMixin):
                     placement_attr=vim.get('placement_attr'),
                     is_default=vim.get('is_default'),
                     status=vim.get('status'),
+                    extra=vim.get('extra'),
                     deleted_at=datetime.min)
                 context.session.add(vim_db)
                 vim_auth_db = nfvo_db.VimAuth(
                     id=uuidutils.generate_uuid(),
                     vim_id=vim.get('id'),
-                    password=vim_cred.pop('password'),
+                    password=vim_cred.pop('password', None),
                     vim_project=vim.get('vim_project'),
                     auth_url=vim.get('auth_url'),
                     auth_cred=vim_cred)
@@ -158,14 +170,15 @@ class NfvoPluginDb(nfvo.NFVOPluginBase, db_base.CommonDbMixin):
                 if 'placement_attr' in vim:
                     vim_db.update(
                         {'placement_attr': vim.get('placement_attr')})
+                if 'extra' in vim:
+                    vim_db.update({'extra': vim.get('extra')})
                 vim_auth_db = (self._model_query(
                     context, nfvo_db.VimAuth).filter(
-                        nfvo_db.VimAuth.vim_id == vim_id).with_lockmode(
-                            'update').one())
+                    nfvo_db.VimAuth.vim_id == vim_id).with_for_update().one())
             except orm_exc.NoResultFound:
                 raise nfvo.VimNotFoundException(vim_id=vim_id)
             vim_auth_db.update({'auth_cred': vim_cred, 'password':
-                                vim_cred.pop('password'), 'vim_project':
+                                vim_cred.pop('password', None), 'vim_project':
                                 vim_project})
             vim_db.update({'updated_at': timeutils.utcnow()})
             self._cos_db_plg.create_event(
@@ -181,7 +194,7 @@ class NfvoPluginDb(nfvo.NFVOPluginBase, db_base.CommonDbMixin):
         with context.session.begin(subtransactions=True):
             try:
                 vim_db = (self._model_query(context, nfvo_db.Vim).filter(
-                    nfvo_db.Vim.id == vim_id).with_lockmode('update').one())
+                    nfvo_db.Vim.id == vim_id).with_for_update().one())
             except orm_exc.NoResultFound:
                 raise nfvo.VimNotFoundException(vim_id=vim_id)
             vim_db.update({'status': status,
@@ -201,7 +214,9 @@ class NfvoPluginDb(nfvo.NFVOPluginBase, db_base.CommonDbMixin):
 
     def _get_default_vim(self, context):
         query = self._model_query(context, nfvo_db.Vim)
-        return query.filter(nfvo_db.Vim.is_default == sql.true()).one()
+        return query.filter(
+            nfvo_db.Vim.tenant_id == context.tenant_id).filter(
+            nfvo_db.Vim.is_default == sql.true()).one()
 
     def get_default_vim(self, context):
         vim_db = self._get_default_vim(context)
