@@ -19,6 +19,7 @@ from oslo_log import log as logging
 from tacker._i18n import _
 from tacker.common.container import kubernetes_utils
 from tacker.common import log
+from tacker.common import utils
 from tacker import context as t_context
 from tacker.extensions import nfvo
 from tacker.keymgr import API as KEYMGR_API
@@ -75,8 +76,8 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
         return auth_cred, file_descriptor
 
     def _create_ssl_ca_file(self, auth_cred):
-        if auth_cred.get('ssl_ca_cert', ''):
-            ca_cert = auth_cred['ssl_ca_cert']
+        ca_cert = utils.none_from_string(auth_cred.get('ssl_ca_cert'))
+        if ca_cert:
             file_descriptor, file_path = \
                 self.kubernetes.create_ca_cert_tmp_file(ca_cert)
             auth_cred['ca_cert_file'] = file_path
@@ -94,7 +95,7 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
             k8s_info = core_api_client.get_api_versions()
             LOG.info(k8s_info)
         except Exception as e:
-            LOG.info('VIM Kubernetes authentication is wrong.')
+            LOG.error('VIM Kubernetes authentication is wrong.')
             # delete temp file
             self.clean_authenticate_vim(auth_dict, file_descriptor)
             raise nfvo.VimUnauthorizedException(message=str(e))
@@ -131,13 +132,17 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
     def register_vim(self, vim_obj):
         """Validate Kubernetes VIM."""
         if 'key_type' in vim_obj['auth_cred']:
-            vim_obj['auth_cred'].pop(u'key_type')
+            vim_obj['auth_cred'].pop('key_type')
         if 'secret_uuid' in vim_obj['auth_cred']:
-            vim_obj['auth_cred'].pop(u'secret_uuid')
+            vim_obj['auth_cred'].pop('secret_uuid')
         self.authenticate_vim(vim_obj)
         self.discover_placement_attr(vim_obj)
         self.encode_vim_auth(vim_obj['id'],
                              vim_obj['auth_cred'])
+        # NOTE(Yao Qibin): To avoid obtaining token multiple times,
+        # the id_token is keeped in auth_cred, which will be deleted here.
+        if 'id_token' in vim_obj['auth_cred']:
+            vim_obj['auth_cred'].pop('id_token')
         LOG.debug('VIM registration completed for %s', vim_obj)
 
     @log.log
@@ -170,9 +175,8 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
                     LOG.debug('VIM key deleted successfully for vim %s',
                               vim_id)
                 except Exception as exception:
-                    LOG.warning('VIM key deletion failed for vim %s due to %s',
-                                vim_id,
-                                exception)
+                    LOG.error('VIM key deletion failed for vim %s due to %s',
+                              vim_id, exception)
                     raise
             else:
                 raise nfvo.VimEncryptKeyError(vim_id=vim_id)
@@ -181,8 +185,8 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
     def encode_vim_auth(self, vim_id, auth):
         """Encode VIM credentials
 
-         Store VIM auth using fernet key encryption
-         """
+        Store VIM auth using fernet key encryption
+        """
         fernet_key, fernet_obj = self.kubernetes.create_fernet_key()
         if ('password' in auth) and (auth['password'] is not None):
             encoded_auth = fernet_obj.encrypt(
@@ -192,10 +196,14 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
             encoded_auth = fernet_obj.encrypt(
                 auth['bearer_token'].encode('utf-8'))
             auth['bearer_token'] = encoded_auth
-        if auth.get('ssl_ca_cert', ''):
+        if utils.none_from_string(auth.get('ssl_ca_cert')):
             encoded_auth = fernet_obj.encrypt(
                 auth['ssl_ca_cert'].encode('utf-8'))
             auth['ssl_ca_cert'] = encoded_auth
+        if 'client_secret' in auth:
+            encoded_auth = fernet_obj.encrypt(
+                auth["client_secret"].encode("utf-8"))
+            auth["client_secret"] = encoded_auth
 
         if CONF.k8s_vim.use_barbican:
             try:
@@ -209,9 +217,8 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
                 LOG.debug('VIM auth successfully stored for vim %s',
                           vim_id)
             except Exception as exception:
-                LOG.warning('VIM key creation failed for vim %s due to %s',
-                            vim_id,
-                            exception)
+                LOG.error('VIM key creation failed for vim %s due to %s',
+                          vim_id, exception)
                 raise
         else:
             raise nfvo.VimEncryptKeyError(vim_id=vim_id)
