@@ -12,13 +12,10 @@
 
 import abc
 
-import six
-
 from tacker.api import extensions
 
 
-@six.add_metaclass(abc.ABCMeta)
-class VnffgAbstractDriver(extensions.PluginInterface):
+class VnffgAbstractDriver(extensions.PluginInterface, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_type(self):
@@ -41,7 +38,8 @@ class VnffgAbstractDriver(extensions.PluginInterface):
         pass
 
     @abc.abstractmethod
-    def update_chain(self, chain_id, fc_ids, vnfs, symmetrical=False,
+    def update_chain(self, chain_id, fc_ids, vnfs,
+                     symmetrical=False,
                      auth_attr=None):
         """Update service function chain"""
         pass
@@ -52,14 +50,12 @@ class VnffgAbstractDriver(extensions.PluginInterface):
         pass
 
     @abc.abstractmethod
-    def create_flow_classifier(self, name, fc, symmetrical=False,
-                               auth_attr=None):
+    def create_flow_classifier(self, name, fc, auth_attr=None):
         """Create flow classifier and returns an ID"""
         pass
 
     @abc.abstractmethod
-    def update_flow_classifier(self, fc_id, fc, symmetrical=False,
-                               auth_attr=None):
+    def update_flow_classifier(self, fc_id, fc, auth_attr=None):
         """Update flow classifier"""
         pass
 
