@@ -91,18 +91,20 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
 
     @staticmethod
     def validate_keystone_auth_url(auth_url, verify):
+        # NOTE(h-asahina): `verify` will be used as an arg of session to
+        # validate certificate
         keystone_obj = keystone.Keystone()
         auth_url = utils.get_auth_url_v3(auth_url)
         try:
-            return keystone_obj.get_version(auth_url, verify)
+            keystone_obj.get_version(auth_url, verify)
         except Exception as e:
-            LOG.error('Keystone Auth URL invalid')
+            LOG.error(f'Validation Failed for Keystone auth_url: {auth_url}')
             raise nfvo.VimConnectionException(message=str(e))
 
     def get_auth_dict(self, context):
         auth = CONF.keystone_authtoken
         auth_url = utils.get_auth_url_v3(auth.auth_url)
-        self.validate_keystone_auth_url(auth_url, 'True')
+        self.validate_keystone_auth_url(auth_url=auth_url, verify=True)
         return {
             'auth_url': auth_url,
             'token': context.auth_token,
@@ -161,7 +163,21 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
             # is not updatable. so no need to consider it
             if 'auth_cred' in update_args:
                 auth_cred = update_args['auth_cred']
-                if ('username' in auth_cred) and ('password' in auth_cred)\
+                if 'oidc_token_url' in auth_cred:
+                    # update oidc info, and remove bearer_token if exists
+                    vim_obj['auth_cred']['oidc_token_url'] = auth_cred.get(
+                        'oidc_token_url')
+                    vim_obj['auth_cred']['username'] = auth_cred.get(
+                        'username')
+                    vim_obj['auth_cred']['password'] = auth_cred.get(
+                        'password')
+                    vim_obj['auth_cred']['client_id'] = auth_cred.get(
+                        'client_id')
+                    vim_obj['auth_cred']['client_secret'] = auth_cred.get(
+                        'client_secret')
+                    if 'bearer_token' in vim_obj['auth_cred']:
+                        vim_obj['auth_cred'].pop('bearer_token')
+                elif ('username' in auth_cred) and ('password' in auth_cred)\
                         and (auth_cred['password'] is not None):
                     # update new username and password, remove bearer_token
                     # if it exists in the old vim
@@ -169,15 +185,27 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                     vim_obj['auth_cred']['password'] = auth_cred['password']
                     if 'bearer_token' in vim_obj['auth_cred']:
                         vim_obj['auth_cred'].pop('bearer_token')
+                    if 'oidc_token_url' in vim_obj['auth_cred']:
+                        vim_obj['auth_cred'].pop('oidc_token_url')
+                    if 'client_id' in vim_obj['auth_cred']:
+                        vim_obj['auth_cred'].pop('client_id')
+                    if 'client_secret' in vim_obj['auth_cred']:
+                        vim_obj['auth_cred'].pop('client_secret')
                 elif 'bearer_token' in auth_cred:
                     # update bearer_token, remove username and password
                     # if they exist in the old vim
                     vim_obj['auth_cred']['bearer_token'] =\
                         auth_cred['bearer_token']
-                    if ('username' in vim_obj['auth_cred']) and\
-                            ('password' in vim_obj['auth_cred']):
+                    if 'username' in vim_obj['auth_cred']:
                         vim_obj['auth_cred'].pop('username')
+                    if 'password' in vim_obj['auth_cred']:
                         vim_obj['auth_cred'].pop('password')
+                    if 'oidc_token_url' in vim_obj['auth_cred']:
+                        vim_obj['auth_cred'].pop('oidc_token_url')
+                    if 'client_id' in vim_obj['auth_cred']:
+                        vim_obj['auth_cred'].pop('client_id')
+                    if 'client_secret' in vim_obj['auth_cred']:
+                        vim_obj['auth_cred'].pop('client_secret')
                 if 'ssl_ca_cert' in auth_cred:
                     # update new ssl_ca_cert
                     vim_obj['auth_cred']['ssl_ca_cert'] =\
@@ -206,7 +234,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                                 vim_id, ex)
             return vim_obj
         except Exception as ex:
-            LOG.debug("Got exception when update_vim %s due to %s",
+            LOG.error("Got exception when update_vim %s due to %s",
                       vim_id, ex)
             with excutils.save_and_reraise_exception():
                 if new_auth_created:
@@ -570,7 +598,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
 
         f = fernet.Fernet(vim_key)
         if not f:
-            LOG.warning('Unable to decode VIM auth')
+            LOG.error('Unable to decode VIM auth')
             raise nfvo.VimNotFoundException(vim_id=vim_id)
         return f.decrypt(cred).decode('utf-8')
 
@@ -580,9 +608,9 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         LOG.debug('Attempting to open key file for vim id %s', vim_id)
         try:
             with open(key_file, 'r') as f:
-                    return f.read()
+                return f.read()
         except Exception:
-            LOG.warning('VIM id invalid or key not found for  %s', vim_id)
+            LOG.error('VIM id invalid or key not found for  %s', vim_id)
             raise nfvo.VimKeyNotFoundException(vim_id=vim_id)
 
     def _vim_resource_name_to_id(self, context, resource, name, vnf_id):
@@ -764,7 +792,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         vnfd_dict = dict()
         for node_name, node_val in \
                 (nsd_dict['topology_template']['node_templates']).items():
-            if node_val.get('type') not in vnfds.keys():
+            if node_val.get('type') not in vnfds:
                 continue
             vnfd_name = vnfds[node_val.get('type')]
             if not vnfd_dict.get(vnfd_name):
@@ -880,7 +908,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                     if value['get_input'] in paramvalues:
                         original[key] = paramvalues[value['get_input']]
                     else:
-                        LOG.debug('Key missing Value: %s', key)
+                        LOG.error('Key missing Value: %s', key)
                         raise cs.InputValuesMissing(key=key)
                 else:
                     self._update_params(value, paramvalues)
