@@ -13,21 +13,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
+from collections import abc
 import re
-
+import sys
 
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
+from oslo_policy import opts
 from oslo_policy import policy
 from oslo_utils import excutils
 from oslo_utils import importutils
-import six
 
-from tacker._i18n import _, _LE, _LW
+from tacker._i18n import _
 from tacker.api.v1 import attributes
 from tacker.common import exceptions
+from tacker import policies
 
 
 LOG = logging.getLogger(__name__)
@@ -35,6 +36,12 @@ LOG = logging.getLogger(__name__)
 _ENFORCER = None
 ADMIN_CTX_POLICY = 'context_is_admin'
 
+# TODO(gmann): Remove setting the default value of config policy_file
+# once oslo_policy change the default value to 'policy.yaml'.
+# https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+DEFAULT_POLICY_FILE = 'policy.yaml'
+opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
+
 
 def reset():
     global _ENFORCER
@@ -49,7 +56,29 @@ def init(conf=cfg.CONF, policy_file=None):
     global _ENFORCER
     if not _ENFORCER:
         _ENFORCER = policy.Enforcer(conf, policy_file=policy_file)
-        _ENFORCER.load_rules(True)
+        register_rules(_ENFORCER)
+        _ENFORCER.load_rules()
+
+
+def authorize(context, action, target, do_raise=True, exc=None):
+
+    init()
+    credentials = context.to_policy_values()
+    if not exc:
+        exc = exceptions.PolicyNotAuthorized
+    try:
+        result = _ENFORCER.authorize(action, target, credentials,
+                                     do_raise=do_raise, exc=exc, action=action)
+    except policy.PolicyNotRegistered:
+        with excutils.save_and_reraise_exception():
+            LOG.error('Policy not registered')
+    except Exception:
+        with excutils.save_and_reraise_exception():
+            LOG.error('Policy check for %(action)s failed with credentials '
+                      '%(credentials)s',
+                      {'action': action, 'credentials': credentials})
+
+    return result
 
 
 def refresh(policy_file=None):
@@ -100,7 +129,7 @@ def _is_attribute_explicitly_set(attribute_name, resource, target, action):
 def _should_validate_sub_attributes(attribute, sub_attr):
     """Verify that sub-attributes are iterable and should be validated."""
     validate = attribute.get('validate')
-    return (validate and isinstance(sub_attr, collections.Iterable) and
+    return (validate and isinstance(sub_attr, abc.Iterable) and
             any([k.startswith('type:dict') and
                  v for (k, v) in validate.items()]))
 
@@ -113,8 +142,7 @@ def _build_subattr_match_rule(attr_name, attr, action, target):
     validate = attr['validate']
     key = list(filter(lambda k: k.startswith('type:dict'), validate.keys()))
     if not key:
-        LOG.warning(_LW("Unable to find data type descriptor "
-                        "for attribute %s"),
+        LOG.warning("Unable to find data type descriptor for attribute %s",
                     attr_name)
         return
     data = validate[key[0]]
@@ -182,7 +210,7 @@ def _build_match_rule(action, target, pluralized):
 # This check is registered as 'tenant_id' so that it can override
 # GenericCheck which was used for validating parent resource ownership.
 # This will prevent us from having to handling backward compatibility
-# for policy.json
+# for policy.yaml
 # TODO(salv-orlando): Reinstate GenericCheck for simple tenant_id checks
 @policy.register('tenant_id')
 class OwnerCheck(policy.Check):
@@ -200,7 +228,7 @@ class OwnerCheck(policy.Check):
             self.target_field = re.findall(r'^\%\((.*)\)s$',
                                            match)[0]
         except IndexError:
-            err_reason = (_("Unable to identify a target field from:%s. "
+            err_reason = (_("Unable to identify a target field from: %s. "
                             "Match should be in the form %%(<field_name>)s") %
                           match)
             LOG.exception(err_reason)
@@ -230,7 +258,7 @@ class OwnerCheck(policy.Check):
                               self.target_field)
             else:
                 # If we are here split failed with both separators
-                err_reason = (_("Unable to find resource name in %s") %
+                err_reason = ("Unable to find resource name in %s" %
                               self.target_field)
                 LOG.error(err_reason)
                 raise exceptions.PolicyCheckError(
@@ -239,8 +267,8 @@ class OwnerCheck(policy.Check):
             parent_foreign_key = attributes.RESOURCE_FOREIGN_KEYS.get(
                 "%ss" % parent_res, None)
             if not parent_foreign_key:
-                err_reason = (_("Unable to verify match:%(match)s as the "
-                                "parent resource: %(res)s was not found") %
+                err_reason = ("Unable to verify match:%(match)s as the "
+                              "parent resource: %(res)s was not found" %
                               {'match': self.match, 'res': parent_res})
                 LOG.error(err_reason)
                 raise exceptions.PolicyCheckError(
@@ -272,11 +300,10 @@ class OwnerCheck(policy.Check):
                 raise db_exc.RetryRequest(e)
             except Exception:
                 with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE('Policy check error while calling %s!'),
-                                  f)
+                    LOG.exception('Policy check error while calling %s!', f)
         match = self.match % target
         if self.kind in creds:
-            return match == six.text_type(creds[self.kind])
+            return match == str(creds[self.kind])
         return False
 
 
@@ -295,7 +322,7 @@ class FieldCheck(policy.Check):
             attr = attributes.RESOURCE_ATTRIBUTE_MAP[resource][field]
             conv_func = attr['convert_to']
         except KeyError:
-            conv_func = lambda x: x
+            conv_func = lambda x: x  # noqa: E731
 
         self.field = field
         self.value = conv_func(value)
@@ -401,7 +428,7 @@ def enforce(context, action, target, plugin=None, pluralized=None):
     except policy.PolicyNotAuthorized:
         with excutils.save_and_reraise_exception():
             log_rule_list(rule)
-            LOG.debug("Failed policy check for '%s'", action)
+            LOG.error("Failed policy check for '%s'", action)
     return result
 
 
@@ -409,7 +436,35 @@ def check_is_admin(context):
     """Verify context has admin rights according to policy settings."""
     init()
     # the target is user-self
-    credentials = context.to_dict()
-    if ADMIN_CTX_POLICY not in _ENFORCER.rules:
+    credentials = context.to_policy_values()
+    try:
+        return _ENFORCER.authorize(ADMIN_CTX_POLICY, credentials, credentials)
+    except policy.PolicyNotRegistered:
         return False
-    return _ENFORCER.enforce(ADMIN_CTX_POLICY, credentials, credentials)
+
+
+def register_rules(enforcer):
+    enforcer.register_defaults(policies.list_rules())
+
+
+def get_enforcer():
+    # NOTE(amotoki): This was borrowed from nova/policy.py.
+    # This method is for use by oslo.policy CLI scripts. Those scripts need the
+    # 'output-file' and 'namespace' options, but having those in sys.argv means
+    # loading the tacker config options will fail as those are not expected to
+    # be present. So we pass in an arg list with those stripped out.
+    conf_args = []
+    # Start at 1 because cfg.CONF expects the equivalent of sys.argv[1:]
+    i = 1
+    while i < len(sys.argv):
+        if sys.argv[i].strip('-') in ['namespace', 'output-file']:
+            i += 2
+            continue
+        conf_args.append(sys.argv[i])
+        i += 1
+
+    # 'project' must be 'tacker' so that get_enforcer looks at
+    # /etc/tacker/policy.yaml by default.
+    cfg.CONF(conf_args, project='tacker')
+    init()
+    return _ENFORCER
