@@ -16,18 +16,16 @@
 import os
 import pickle
 import subprocess
-from urllib.parse import urlparse
-import urllib.request as urllib2
 
 from oslo_log import log as logging
 from oslo_utils import uuidutils
-import yaml
 
 from tacker.sol_refactored.common import config
 from tacker.sol_refactored.common import exceptions as sol_ex
 from tacker.sol_refactored.common import lcm_op_occ_utils as lcmocc_utils
 from tacker.sol_refactored.common import vim_utils
 from tacker.sol_refactored.common import vnf_instance_utils as inst_utils
+from tacker.sol_refactored.infra_drivers.kubernetes import helm
 from tacker.sol_refactored.infra_drivers.kubernetes import kubernetes
 from tacker.sol_refactored.infra_drivers.openstack import openstack
 from tacker.sol_refactored.nfvo import nfvo_client
@@ -68,13 +66,10 @@ class VnfLcmDriverV2(object):
         grant_req = objects.GrantRequestV1(
             vnfInstanceId=inst.id,
             vnfLcmOpOccId=lcmocc.id,
+            vnfdId=inst.vnfdId,
             operation=lcmocc.operation,
             isAutomaticInvocation=lcmocc.isAutomaticInvocation
         )
-        if lcmocc.operation == v2fields.LcmOperationType.CHANGE_VNFPKG:
-            grant_req.vnfdId = lcmocc.operationParams.get('vnfdId')
-        else:
-            grant_req.vnfdId = inst.vnfdId
         grant_req._links = objects.GrantRequestV1_Links(
             vnfLcmOpOcc=objects.Link(
                 href=lcmocc_utils.lcmocc_href(lcmocc.id, self.endpoint)),
@@ -128,6 +123,17 @@ class VnfLcmDriverV2(object):
             LOG.debug("execute %s failed: %s", operation, out.stderr)
             msg = "{} failed: {}".format(operation, out.stderr)
             raise sol_ex.MgmtDriverExecutionFailed(sol_detail=msg)
+        else:
+            try:
+                output = pickle.loads(out.stdout)
+                if isinstance(output, dict) and 'vnf_instance' in output:
+                    _inst = objects.VnfInstanceV2.from_dict(
+                        output['vnf_instance'])
+                    inst.__dict__.update(_inst.__dict__)
+            except EOFError:
+                pass
+            except pickle.UnpicklingError:
+                pass
 
         LOG.debug("execute %s of %s success.", operation, script)
 
@@ -379,7 +385,7 @@ class VnfLcmDriverV2(object):
             # As the controller does for req.vimConnectionInfo, if accessInfo
             # or interfaceInfo is not specified, get them from VIM DB.
             # vimId must be in VIM DB.
-            res_vim_infos = grant.vimConnectioninfo
+            res_vim_infos = grant.vimConnectionInfo
             for key, res_vim_info in res_vim_infos.items():
                 if not (res_vim_info.obj_attr_is_set('accessInfo') and
                         res_vim_info.obj_attr_is_set('interfaceInfo')):
@@ -399,6 +405,34 @@ class VnfLcmDriverV2(object):
 
         inst.vimConnectionInfo = vim_infos
 
+        # vimType dependent parameter check
+        # NOTE: controller cannot check because vim_info is not identified
+        # to here, although it is better to check in controller.
+        if lcmocc.operationState == v2fields.LcmOperationStateType.STARTING:
+            vim_info = inst_utils.select_vim_info(vim_infos)
+            if vim_info.vimType == "kubernetes":
+                if 'endpoint' not in vim_info.interfaceInfo:
+                    detail = "Required attribute missing in vimConnectionInfo"
+                    raise sol_ex.SolValidationError(detail=detail)
+                if (not req.get('additionalParams', {}).get(
+                        'lcm-kubernetes-def-files')):
+                    raise sol_ex.SolValidationError(
+                        detail="'lcm-kubernetes-def-files' must be specified")
+            elif vim_info.vimType == "ETSINFV.HELM.V_3":
+                if ('endpoint' not in vim_info.interfaceInfo or
+                        'ssl_ca_cert' not in vim_info.interfaceInfo or
+                        'bearer_token' not in vim_info.accessInfo):
+                    detail = "Required attribute missing in vimConnectionInfo"
+                    raise sol_ex.SolValidationError(detail=detail)
+                if (not req.get('additionalParams', {}).get(
+                        'helm_chart_path')):
+                    raise sol_ex.SolValidationError(
+                        detail="'helm_chart_path' must be specified")
+                if (not req.get('additionalParams', {}).get(
+                        'helm_value_names')):
+                    raise sol_ex.SolValidationError(
+                        detail="'helm_value_names' must be specified")
+
     def instantiate_process(self, context, lcmocc, inst, grant_req,
             grant, vnfd):
         req = lcmocc.operationParams
@@ -409,9 +443,12 @@ class VnfLcmDriverV2(object):
         if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
             driver = openstack.Openstack()
             driver.instantiate(req, inst, grant_req, grant, vnfd)
-        elif vim_info.vimType == 'kubernetes':  # k8s
+        elif vim_info.vimType == 'kubernetes':
             driver = kubernetes.Kubernetes()
             driver.instantiate(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.instantiate(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
@@ -425,8 +462,14 @@ class VnfLcmDriverV2(object):
         if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
             driver = openstack.Openstack()
             driver.instantiate_rollback(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'kubernetes':
+            driver = kubernetes.Kubernetes()
+            driver.instantiate_rollback(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.instantiate_rollback(req, inst, grant_req, grant, vnfd)
         else:
-            # only support openstack at the moment
+            # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
 
     def _make_res_def_for_remove_vnfcs(self, inst_info, inst_vnfcs,
@@ -564,9 +607,12 @@ class VnfLcmDriverV2(object):
         if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
             driver = openstack.Openstack()
             driver.terminate(req, inst, grant_req, grant, vnfd)
-        elif vim_info.vimType == 'kubernetes':  # k8s
+        elif vim_info.vimType == 'kubernetes':
             driver = kubernetes.Kubernetes()
             driver.terminate(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.terminate(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
@@ -677,8 +723,14 @@ class VnfLcmDriverV2(object):
         if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
             driver = openstack.Openstack()
             driver.scale(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'kubernetes':
+            driver = kubernetes.Kubernetes()
+            driver.scale(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.scale(req, inst, grant_req, grant, vnfd)
         else:
-            # only support openstack at the moment
+            # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
 
     def scale_rollback(self, context, lcmocc, inst, grant_req,
@@ -691,8 +743,14 @@ class VnfLcmDriverV2(object):
         if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
             driver = openstack.Openstack()
             driver.scale_rollback(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'kubernetes':
+            driver = kubernetes.Kubernetes()
+            driver.scale_rollback(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.scale_rollback(req, inst, grant_req, grant, vnfd)
         else:
-            # only support openstack at the moment
+            # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
 
     def _modify_from_vnfd_prop(self, inst, vnfd_prop, attr):
@@ -870,8 +928,14 @@ class VnfLcmDriverV2(object):
         if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
             driver = openstack.Openstack()
             driver.heal(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'kubernetes':
+            driver = kubernetes.Kubernetes()
+            driver.heal(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.heal(req, inst, grant_req, grant, vnfd)
         else:
-            # only support openstack at the moment
+            # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
 
     def change_ext_conn_grant(self, grant_req, req, inst, vnfd):
@@ -966,159 +1030,6 @@ class VnfLcmDriverV2(object):
             # only support openstack at the moment
             raise sol_ex.SolException(sol_detail='not support vim type')
 
-    def change_vnfpkg_grant(self, grant_req, req, inst, vnfd):
-        inst_info = inst.instantiatedVnfInfo
-        grant_req.flavourId = inst_info.flavourId
-        if req.additionalParams.get('vdu_params'):
-            target_vdu_ids = [
-                vdu_param.get(
-                    'vdu_id') for vdu_param in req.additionalParams.get(
-                    'vdu_params')]
-        else:
-            if inst_info.obj_attr_is_set('vnfcResourceInfo'):
-                target_vdu_ids = [inst_vnc.vduId for inst_vnc in
-                                  inst_info.vnfcResourceInfo]
-
-        if req.additionalParams.get('upgrade_type') == 'RollingUpdate':
-            update_reses = []
-            add_reses = []
-            remove_reses = []
-            if inst_info.obj_attr_is_set('vnfcResourceInfo'):
-                for inst_vnc in inst_info.vnfcResourceInfo:
-                    if inst_vnc.vduId in target_vdu_ids:
-                        vdu_res_id = uuidutils.generate_uuid()
-                        res_def = objects.ResourceDefinitionV1(
-                            id=vdu_res_id,
-                            type='COMPUTE',
-                            resourceTemplateId=inst_vnc.vduId)
-                        update_reses.append(res_def)
-                        nodes = vnfd.get_vdu_nodes(inst_info.flavourId)
-                        vdu_storage_names = vnfd.get_vdu_storages(
-                            nodes[inst_vnc.vduId])
-                        for vdu_storage_name in vdu_storage_names:
-                            res_def = objects.ResourceDefinitionV1(
-                                id=_make_combination_id(
-                                    vdu_storage_name, vdu_res_id),
-                                type='STORAGE',
-                                resourceTemplateId=vdu_storage_name)
-                            add_reses.append(res_def)
-                        if inst_vnc.obj_attr_is_set('storageResourceIds'):
-                            inst_stor_info = (
-                                inst_info.virtualStorageResourceInfo)
-                            for str_info in inst_stor_info:
-                                if str_info.id in inst_vnc.storageResourceIds:
-                                    res_def = objects.ResourceDefinitionV1(
-                                        id=uuidutils.generate_uuid(),
-                                        type='STORAGE',
-                                        resourceTemplateId=(
-                                            str_info.virtualStorageDescId),
-                                        resource=str_info.storageResource)
-                                    remove_reses.append(res_def)
-            if update_reses:
-                grant_req.updateResources = update_reses
-
-            if add_reses:
-                grant_req.addResources = add_reses
-
-            if remove_reses:
-                grant_req.removeResources = remove_reses
-        else:
-            # TODO(YiFeng): Blue-Green type will be supported in Zed release.
-            # not reach here at the moment
-            pass
-
-    def _pre_check_for_change_vnfpkg(self, context, req, inst, vnfd):
-        def _get_file_content(file_path):
-            if ((urlparse(file_path).scheme == 'file') or
-                    (bool(urlparse(file_path).scheme) and
-                     bool(urlparse(file_path).netloc))):
-                with urllib2.urlopen(file_path) as file_object:
-                    file_content = file_object.read()
-            else:
-                with open(file_path, 'rb') as file_object:
-                    file_content = file_object.read()
-            return file_content
-
-        vnf_artifact_files = vnfd.get_vnf_artifact_files()
-        if req.additionalParams.get('lcm-kubernetes-def-files') is None:
-            target_k8s_files = inst.metadata.get('lcm-kubernetes-def-files')
-        else:
-            target_k8s_files = []
-            new_file_paths = req.additionalParams.get(
-                'lcm-kubernetes-def-files')
-            old_vnfd = self.nfvo_client.get_vnfd(
-                context=context, vnfd_id=inst.vnfdId, all_contents=False)
-            old_file_paths = inst.metadata.get('lcm-kubernetes-def-files')
-
-            for new_file_path in new_file_paths:
-                new_file_infos = [
-                    {"kind": content.get('kind'),
-                     "name": content.get('metadata', {}).get('name', '')}
-                    for content in list(yaml.safe_load_all(
-                        _get_file_content(os.path.join(
-                            vnfd.csar_dir, new_file_path))))]
-                for old_file_path in old_file_paths:
-                    find_flag = False
-                    old_file_infos = [
-                        {"kind": content.get('kind'),
-                         "name": content.get('metadata', {}).get('name', '')}
-                        for content in list(yaml.safe_load_all(
-                            _get_file_content(os.path.join(
-                                old_vnfd.csar_dir, old_file_path))))]
-                    resources = [info for info in old_file_infos
-                                 if info in new_file_infos]
-                    if len(resources) != 0:
-                        if len(resources) != len(old_file_infos):
-                            raise sol_ex.UnmatchedFileException(
-                                new_file_path=new_file_path)
-                        if 'Deployment' not in [res.get(
-                                'kind') for res in resources]:
-                            raise sol_ex.UnSupportedKindException(
-                                new_file_path=new_file_path)
-                        old_file_paths.remove(old_file_path)
-                        target_k8s_files.append(new_file_path)
-                        find_flag = True
-                        break
-                    continue
-                if not find_flag:
-                    raise sol_ex.NotFoundUpdateFileException(
-                        new_file_path=new_file_path)
-
-            target_k8s_files.extend(old_file_paths)
-        if set(target_k8s_files).difference(set(vnf_artifact_files)):
-            diff_files = ','.join(list(set(
-                target_k8s_files).difference(set(vnf_artifact_files))))
-            raise sol_ex.CnfDefinitionNotFound(diff_files=diff_files)
-        return target_k8s_files
-
-    def change_vnfpkg_process(
-            self, context, lcmocc, inst, grant_req, grant, vnfd):
-        inst_saved = inst.obj_clone()
-        req = lcmocc.operationParams
-        vim_info = inst_utils.select_vim_info(inst.vimConnectionInfo)
-        if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
-            driver = openstack.Openstack()
-            try:
-                driver.change_vnfpkg(req, inst, grant_req, grant, vnfd)
-            except Exception as ex:
-                lcmocc_utils.update_lcmocc(lcmocc, inst_saved, inst)
-                raise Exception from ex
-        elif vim_info.vimType == 'kubernetes':  # k8s
-            target_k8s_files = self._pre_check_for_change_vnfpkg(
-                context, req, inst, vnfd)
-            update_req = req.obj_clone()
-            update_req.additionalParams[
-                'lcm-kubernetes-def-files'] = target_k8s_files
-            driver = kubernetes.Kubernetes()
-            try:
-                driver.change_vnfpkg(update_req, inst, grant_req, grant, vnfd)
-            except Exception as ex:
-                lcmocc_utils.update_lcmocc(lcmocc, inst_saved, inst)
-                raise Exception from ex
-        else:
-            # should not occur
-            raise sol_ex.SolException(sol_detail='not support vim type')
-
     def change_ext_conn_rollback(self, context, lcmocc, inst, grant_req,
             grant, vnfd):
         req = lcmocc.operationParams
@@ -1130,18 +1041,117 @@ class VnfLcmDriverV2(object):
             # only support openstack at the moment
             raise sol_ex.SolException(sol_detail='not support vim type')
 
+    def change_vnfpkg_grant(self, grant_req, req, inst, vnfd):
+        grant_req.dstVnfdId = req.vnfdId
+        # NOTE: flavourId is not necessary according to the SOL003 spec.
+        # It is for local_nfvo to handle images.
+        grant_req.flavourId = inst.instantiatedVnfInfo.flavourId
+
+        if req.additionalParams['upgrade_type'] == 'RollingUpdate':
+            self._change_vnfpkg_grant_rolling_update(
+                grant_req, req, inst, vnfd)
+        else:
+            # Only support RollingUpdate at the moment.
+            pass
+
+    def _change_vnfpkg_grant_rolling_update(self, grant_req, req, inst, vnfd):
+        inst_info = inst.instantiatedVnfInfo
+        if not inst_info.obj_attr_is_set('vnfcResourceInfo'):
+            return
+
+        vdu_ids = [vdu_param['vdu_id']
+                   for vdu_param in req.additionalParams['vdu_params']]
+        inst_vnfcs = [inst_vnfc
+                      for inst_vnfc in inst_info.vnfcResourceInfo
+                      if inst_vnfc.vduId in vdu_ids]
+
+        add_reses = []
+        rm_reses = []
+        for inst_vnfc in inst_vnfcs:
+            rm_vdu_res_id = uuidutils.generate_uuid()
+            add_vdu_res_id = uuidutils.generate_uuid()
+            self._set_rm_add_reses(rm_reses, add_reses, 'COMPUTE',
+                inst_vnfc.vduId, inst_vnfc.computeResource,
+                rm_vdu_res_id, add_vdu_res_id)
+
+            if inst_vnfc.obj_attr_is_set('storageResourceIds'):
+                for storage_id in inst_vnfc.storageResourceIds:
+                    inst_str = _get_obj_by_id(storage_id,
+                        inst_info.virtualStorageResourceInfo)
+                    str_name = inst_str.virtualStorageDescId
+                    self._set_rm_add_reses(rm_reses, add_reses, 'STORAGE',
+                        str_name, inst_str.storageResource,
+                        _make_combination_id(str_name, rm_vdu_res_id),
+                        _make_combination_id(str_name, add_vdu_res_id))
+
+        if add_reses:
+            grant_req.addResources = add_reses
+        if rm_reses:
+            grant_req.removeResources = rm_reses
+
+    def change_vnfpkg_process(
+            self, context, lcmocc, inst, grant_req, grant, vnfd):
+        req = lcmocc.operationParams
+        if req.obj_attr_is_set('vimConnectionInfo'):
+            self._merge_vim_connection_info(inst, req)
+
+        vim_info = inst_utils.select_vim_info(inst.vimConnectionInfo)
+        if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
+            driver = openstack.Openstack()
+            driver.change_vnfpkg(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'kubernetes':
+            driver = kubernetes.Kubernetes()
+            driver.change_vnfpkg(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.change_vnfpkg(req, inst, grant_req, grant, vnfd)
+        else:
+            # should not occur
+            raise sol_ex.SolException(sol_detail='not support vim type')
+
+        inst.vnfdId = grant_req.dstVnfdId
+
     def change_vnfpkg_rollback(
             self, context, lcmocc, inst, grant_req, grant, vnfd):
         vim_info = inst_utils.select_vim_info(inst.vimConnectionInfo)
         req = lcmocc.operationParams
-        driver = openstack.Openstack()
         if vim_info.vimType == 'ETSINFV.OPENSTACK_KEYSTONE.V_3':
+            driver = openstack.Openstack()
             driver.change_vnfpkg_rollback(
                 req, inst, grant_req, grant, vnfd, lcmocc)
-        elif vim_info.vimType == 'kubernetes':  # k8s
+        elif vim_info.vimType == 'kubernetes':
             driver = kubernetes.Kubernetes()
             driver.change_vnfpkg_rollback(
-                req, inst, grant_req, grant, vnfd, lcmocc)
+                req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.change_vnfpkg_rollback(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
+
+    def sync_db(self, context, vnf_inst, vim_info):
+        # Database synchronization works only when
+        # the vimType is kubernetes or ETSINFV.HELM.V_3
+        if vim_info.vimType == 'kubernetes':
+            driver = kubernetes.Kubernetes()
+            driver.sync_db(context, vnf_inst, vim_info)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.sync_db(context, vnf_inst, vim_info)
+        else:
+            # Only support CNF
+            raise sol_ex.DbSyncNoDiff(
+                "There are no differences in Vnfc resources.")
+
+    def diff_check_inst(self, vnf_inst, vim_info):
+        if vim_info.vimType == 'kubernetes':
+            driver = kubernetes.Kubernetes()
+            driver.diff_check_inst(vnf_inst, vim_info)
+        elif vim_info.vimType == 'ETSINFV.HELM.V_3':
+            driver = helm.Helm()
+            driver.diff_check_inst(vnf_inst, vim_info)
+        else:
+            # Only support CNF
+            raise sol_ex.DbSyncNoDiff(
+                "There are no differences in Vnfc resources.")
