@@ -16,13 +16,16 @@
 from copy import deepcopy
 import datetime
 import iso8601
+import json
 import os
 import webob
 
 from tacker.api.vnflcm.v1.router import VnflcmAPIRouter
+from tacker.common import utils
 from tacker import context
 from tacker.db.db_sqlalchemy import models
 from tacker import objects
+from tacker.objects.change_ext_conn_req import ChangeExtConnRequest
 from tacker.objects import fields
 from tacker.objects.instantiate_vnf_req import ExtManagedVirtualLinkData
 from tacker.objects.instantiate_vnf_req import ExtVirtualLinkData
@@ -39,6 +42,11 @@ import tacker.conf
 CONF = tacker.conf.CONF
 
 
+def return_vnf_interfaces():
+    vnf_interface = 'vnflcm_noop'
+    return vnf_interface
+
+
 def return_default_vim():
     default_vim = {
         'vim_auth': {
@@ -157,6 +165,15 @@ def return_vnf_instance_model(
     return model_obj
 
 
+def return_vnf_instance_delete(
+        instantiated_state=fields.VnfInstanceState.NOT_INSTANTIATED,
+        **updates):
+    data = _model_non_instantiated_vnf_instance(**updates)
+    data['instantiation_state'] = instantiated_state
+    vnf_instance_obj = objects.VnfInstance(**data)
+    return vnf_instance_obj
+
+
 def return_vnf_instance(
         instantiated_state=fields.VnfInstanceState.NOT_INSTANTIATED,
         scale_status=None,
@@ -165,7 +182,15 @@ def return_vnf_instance(
     if instantiated_state == fields.VnfInstanceState.NOT_INSTANTIATED:
         data = _model_non_instantiated_vnf_instance(**updates)
         data['instantiation_state'] = instantiated_state
+        get_instantiated_vnf_info = {
+            'flavour_id': uuidsentinel.flavour_id,
+            'vnf_state': 'STARTED',
+            'instance_id': ''
+        }
+        instantiated_vnf_info = get_instantiated_vnf_info
+        info_data = objects.InstantiatedVnfInfo(**instantiated_vnf_info)
         vnf_instance_obj = objects.VnfInstance(**data)
+        vnf_instance_obj.instantiated_vnf_info = info_data
 
     elif scale_status:
         data = _model_non_instantiated_vnf_instance(**updates)
@@ -179,8 +204,9 @@ def return_vnf_instance(
         }
         instantiated_vnf_info = get_instantiated_vnf_info
 
-        s_status = {"aspect_id": "SP1", "scale_level": 1}
-        scale_status = objects.ScaleInfo(**s_status)
+        if scale_status == "scale_status":
+            s_status = {"aspect_id": "SP1", "scale_level": 1}
+            scale_status = objects.ScaleInfo(**s_status)
 
         instantiated_vnf_info.update(
             {"ext_cp_info": [],
@@ -219,11 +245,22 @@ def return_vnf_instance(
 
 def _instantiated_vnf_links(vnf_instance_id):
     links = {
-        "self": {"href": "/vnflcm/v1/vnf_instances/%s" % vnf_instance_id},
-        "terminate": {"href": "/vnflcm/v1/vnf_instances/%s/terminate" %
-                      vnf_instance_id},
-        "heal": {"href": "/vnflcm/v1/vnf_instances/%s/heal" %
-                 vnf_instance_id}}
+        "self": {"href":
+            '{endpoint}/vnflcm/v1/vnf_instances/{id}'.format(
+                endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                id=vnf_instance_id)},
+        "terminate": {"href":
+            '{endpoint}/vnflcm/v1/vnf_instances/{id}/terminate'.format(
+                endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                id=vnf_instance_id)},
+        "heal": {"href":
+            '{endpoint}/vnflcm/v1/vnf_instances/{id}/heal'.format(
+                endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                id=vnf_instance_id)},
+        "changeExtConn": {"href":
+            '{endpoint}/vnflcm/v1/vnf_instances/{id}/change_ext_conn'
+             .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                 id=vnf_instance_id)}}
 
     return links
 
@@ -236,14 +273,16 @@ def _fake_vnf_instance_not_instantiated_response(
         'vnfProductName': 'Sample VNF',
         '_links': {
             'self': {
-                'href': os.path.join(
-                    '/vnflcm/v1/vnf_instances/',
-                    uuidsentinel.vnf_instance_id)},
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_instances/{id}'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=uuidsentinel.vnf_instance_id)},
             'instantiate': {
-                'href': os.path.join(
-                    '/vnflcm/v1/vnf_instances',
-                    uuidsentinel.vnf_instance_id,
-                    'instantiate')}},
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_instances/{id}/{op}'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=uuidsentinel.vnf_instance_id,
+                    op='instantiate')}},
         'instantiationState': 'NOT_INSTANTIATED',
         'vnfProvider': 'Vnf provider',
         'vnfdId': uuidsentinel.vnfd_id,
@@ -262,14 +301,14 @@ def _fake_vnf_instance_not_instantiated_response(
 
 def fake_vnf_instance_response(
         instantiated_state=fields.VnfInstanceState.NOT_INSTANTIATED,
-        api_version=None, **updates):
+        api_version=None, vimConnectionInfo=[], **updates):
     if instantiated_state == fields.VnfInstanceState.NOT_INSTANTIATED:
         data = _fake_vnf_instance_not_instantiated_response(**updates)
     else:
         data = _fake_vnf_instance_not_instantiated_response(**updates)
         data['_links'] = _instantiated_vnf_links(uuidsentinel.vnf_instance_id)
         data['instantiationState'] = instantiated_state
-        data['vimConnectionInfo'] = []
+        data['vimConnectionInfo'] = vimConnectionInfo
 
         def _instantiated_vnf_info():
             inst_vnf_info = {}
@@ -419,7 +458,7 @@ def get_vnf_instantiation_request_body():
                                     {
                                         "type": "IPV4",
                                         "fixedAddresses": [
-                                            '192.168.11.01',
+                                            '192.168.11.1',
                                             '192.168.21.202'
                                         ],
                                         "subnetId":
@@ -697,7 +736,31 @@ def get_dummy_vim_connection_info():
         'user_domain_name': 'Default', 'username': 'admin'},
         'created_at': '', 'deleted': False, 'deleted_at': '',
         'id': 'fake_id', 'updated_at': '',
-        'vim_id': 'fake_vim_id', 'vim_type': 'openstack'}
+        'vim_id': 'fake_vim_id', 'vim_type': 'openstack', 'extra': {}}
+
+
+def get_dummy_openstack_vim_obj():
+    return {'vim_id': uuidsentinel.vim_id,
+            'vim_name': 'fake_vim',
+            'vim_type': 'openstack',
+            'vim_auth': {
+                'auth_url': 'http://localhost/identity',
+                'password': 'test_pw',
+                'username': 'test_user',
+                'project_name': 'test_project'},
+            'tenant': uuidsentinel.tenant_id}
+
+
+def get_dummy_k8s_vim_obj():
+    return {'vim_id': uuidsentinel.vim_id,
+            'vim_name': 'fake_vim',
+            'vim_type': 'kubernetes',
+            'vim_auth': {
+                'auth_url': 'http://localhost/8443',
+                'password': 'test_pw',
+                'username': 'test_user',
+                'project_name': 'test_project'},
+            'tenant': uuidsentinel.tenant_id}
 
 
 def get_dummy_instantiate_vnf_request(**updates):
@@ -787,10 +850,10 @@ def _get_vnf(**updates):
     return vnf_data
 
 
-def scale_request(type, number_of_steps, is_reverse):
+def scale_request(type, aspect_id, number_of_steps, is_reverse):
     scale_request_data = {
         'type': type,
-        'aspect_id': "SP1",
+        'aspect_id': aspect_id,
         'number_of_steps': number_of_steps,
         'scale_level': 1,
         'additional_params': {"is_reverse": is_reverse},
@@ -839,12 +902,41 @@ def vnf_scale():
         vim_id=uuidsentinel.vim_id)
 
 
-def vnflcm_rollback(error_point=7):
+def vnflcm_scale_in_cnf():
+    dt = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
     return objects.VnfLcmOpOcc(
-        state_entered_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                             tzinfo=iso8601.UTC),
-        start_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC),
+        state_entered_time=dt,
+        start_time=dt,
+        vnf_instance_id=uuidsentinel.vnf_instance_id,
+        operation='SCALE',
+        operation_state='STARTING',
+        is_automatic_invocation=False,
+        operation_params='{"type": "SCALE_IN", "aspect_id": "vdu1_aspect"}',
+        error_point=1,
+        id=constants.UUID,
+        created_at=dt)
+
+
+def vnflcm_scale_out_cnf():
+    dt = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
+    return objects.VnfLcmOpOcc(
+        state_entered_time=dt,
+        start_time=dt,
+        vnf_instance_id=uuidsentinel.vnf_instance_id,
+        operation='SCALE',
+        operation_state='STARTING',
+        is_automatic_invocation=False,
+        operation_params='{"type": "SCALE_OUT", "aspect_id": "vdu1_aspect"}',
+        error_point=1,
+        id=constants.UUID,
+        created_at=dt)
+
+
+def vnflcm_rollback(error_point=7):
+    dt = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
+    return objects.VnfLcmOpOcc(
+        state_entered_time=dt,
+        start_time=dt,
         vnf_instance_id=uuidsentinel.vnf_instance_id,
         operation='SCALE',
         operation_state='FAILED_TEMP',
@@ -852,16 +944,14 @@ def vnflcm_rollback(error_point=7):
         operation_params='{"type": "SCALE_OUT", "aspect_id": "SP1"}',
         error_point=error_point,
         id=constants.UUID,
-        created_at=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC))
+        created_at=dt)
 
 
 def vnflcm_rollback_insta(error_point=7):
+    dt = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
     return objects.VnfLcmOpOcc(
-        state_entered_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                             tzinfo=iso8601.UTC),
-        start_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC),
+        state_entered_time=dt,
+        start_time=dt,
         vnf_instance_id=uuidsentinel.vnf_instance_id,
         operation='INSTANTIATE',
         operation_state='FAILED_TEMP',
@@ -869,16 +959,129 @@ def vnflcm_rollback_insta(error_point=7):
         operation_params='{}',
         error_point=error_point,
         id=constants.UUID,
-        created_at=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC))
+        created_at=dt)
 
 
-def vnflcm_rollback_active():
+def vnflcm_cancel_insta(error_point=7):
+    default_datetime = datetime.datetime(
+        2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
+    ext_link_port_info = objects.ExtLinkPortInfo(
+        resource_handle=objects.ResourceHandle(
+            resource_id="109f5049-b51e-409a-9a99-d740ba5f3acb",
+            vim_level_resource_type="LINKPORT"),
+        cp_instance_id="f5c68d94-5736-4e38-ade5-c9462514f8b9",
+        id="1d868d02-ecd4-4402-8e6b-54e77ebdcc28")
+    changed_ext_connectivity_values = objects.ExtVirtualLinkInfo(
+        id=constants.UUID,
+        resource_handle=objects.ResourceHandle(
+            vim_connection_id=constants.UUID,
+            resource_id=constants.UUID,
+            vim_level_resource_type="OS::Neutron::Net"),
+        ext_link_ports=[ext_link_port_info]
+    )
+    return objects.VnfLcmOpOcc(
+        state_entered_time=default_datetime,
+        start_time=default_datetime,
+        vnf_instance_id=uuidsentinel.vnf_instance_id,
+        operation='INSTANTIATE',
+        operation_state='PROCESSING',
+        is_automatic_invocation=False,
+        operation_params='{}',
+        error_point=error_point,
+        id=constants.UUID,
+        grant_id=constants.UUID,
+        created_at=default_datetime,
+        changed_ext_connectivity=[changed_ext_connectivity_values])
+
+
+def vnflcm_fail_insta(error_point=7):
+    default_datetime = datetime.datetime(
+        2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
+    ext_link_port_info = objects.ExtLinkPortInfo(
+        resource_handle=objects.ResourceHandle(
+            resource_id="109f5049-b51e-409a-9a99-d740ba5f3acb",
+            vim_level_resource_type="LINKPORT"),
+        cp_instance_id="f5c68d94-5736-4e38-ade5-c9462514f8b9",
+        id="1d868d02-ecd4-4402-8e6b-54e77ebdcc28")
+    changed_ext_connectivity_values = objects.ExtVirtualLinkInfo(
+        id=constants.UUID,
+        resource_handle=objects.ResourceHandle(
+            vim_connection_id=constants.UUID,
+            resource_id=constants.UUID,
+            vim_level_resource_type="OS::Neutron::Net"),
+        ext_link_ports=[ext_link_port_info]
+    )
+    return objects.VnfLcmOpOcc(
+        state_entered_time=default_datetime,
+        start_time=default_datetime,
+        vnf_instance_id=uuidsentinel.vnf_instance_id,
+        operation='INSTANTIATE',
+        operation_state='FAILED_TEMP',
+        is_automatic_invocation=False,
+        operation_params='{}',
+        error_point=error_point,
+        id=constants.UUID,
+        grant_id=constants.UUID,
+        created_at=default_datetime,
+        changed_ext_connectivity=[changed_ext_connectivity_values])
+
+
+def vnflcm_fail_check_added_params(error_point=7):
+    ext_link_port_info = objects.ExtLinkPortInfo(
+        resource_handle=objects.ResourceHandle(
+            resource_id="109f5049-b51e-409a-9a99-d740ba5f3acb",
+            vim_level_resource_type="LINKPORT"),
+        cp_instance_id="f5c68d94-5736-4e38-ade5-c9462514f8b9",
+        id="1d868d02-ecd4-4402-8e6b-54e77ebdcc28")
+    changed_ext_connectivity_values = objects.ExtVirtualLinkInfo(
+        id=constants.UUID,
+        resource_handle=objects.ResourceHandle(
+            vim_connection_id=constants.UUID,
+            resource_id=constants.UUID,
+            vim_level_resource_type="OS::Neutron::Net"),
+        ext_link_ports=[ext_link_port_info]
+    )
+    vim_connection_info = {"id": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+             "vim_id": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+             "vim_type": 'openstack',
+             "access_info": {"key1": 'value1', "key2": 'value2'}}
+    changed_info_values = objects.VnfInfoModifications(
+        vnf_instance_name="fake_name",
+        vnf_instance_description="fake_vnf_instance_description",
+        metadata={'key': 'value'},
+        vim_connection_info=[VimConnectionInfo(**vim_connection_info)],
+        vim_connection_info_delete_ids=[
+            'f8c35bd0-4d67-4436-9f11-14b8a84c92bb'],
+        vnf_pkg_id='f26f181d-7891-4720-b022-b074ec1733ef',
+        vnfd_id="f26f181d-7891-4720-b022-b074ec1733ef",
+        vnf_provider="fake_vnf_provider",
+        vnf_product_name="fake_vnf_product_name",
+        vnf_software_version="fake_vnf_software_version",
+        vnfd_version="fake_vnfd_version")
     return objects.VnfLcmOpOcc(
         state_entered_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
                                              tzinfo=iso8601.UTC),
         start_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
                                      tzinfo=iso8601.UTC),
+        vnf_instance_id=constants.UUID,
+        operation='INSTANTIATE',
+        operation_state='FAILED_TEMP',
+        is_automatic_invocation=False,
+        operation_params='{}',
+        error_point=error_point,
+        id=constants.UUID,
+        grant_id=constants.UUID,
+        created_at=datetime.datetime(2000, 1, 1, 1, 1, 1,
+                                     tzinfo=iso8601.UTC),
+        changed_ext_connectivity=[changed_ext_connectivity_values],
+        changed_info=changed_info_values)
+
+
+def vnflcm_rollback_active():
+    dt = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
+    return objects.VnfLcmOpOcc(
+        state_entered_time=dt,
+        start_time=dt,
         vnf_instance_id=uuidsentinel.vnf_instance_id,
         operation='SCALE',
         operation_state='ACTIVE',
@@ -886,16 +1089,14 @@ def vnflcm_rollback_active():
         operation_params='{"type": "SCALE_OUT", "aspect_id": "SP1"}',
         error_point=7,
         id=constants.UUID,
-        created_at=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC))
+        created_at=dt)
 
 
 def vnflcm_rollback_ope():
+    dt = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
     return objects.VnfLcmOpOcc(
-        state_entered_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                             tzinfo=iso8601.UTC),
-        start_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC),
+        state_entered_time=dt,
+        start_time=dt,
         vnf_instance_id=uuidsentinel.vnf_instance_id,
         operation='HEAL',
         operation_state='FAILED_TEMP',
@@ -903,16 +1104,14 @@ def vnflcm_rollback_ope():
         operation_params='{}',
         error_point=7,
         id=constants.UUID,
-        created_at=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC))
+        created_at=dt)
 
 
 def vnflcm_rollback_scale_in():
+    dt = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
     return objects.VnfLcmOpOcc(
-        state_entered_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                             tzinfo=iso8601.UTC),
-        start_time=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC),
+        state_entered_time=dt,
+        start_time=dt,
         vnf_instance_id=uuidsentinel.vnf_instance_id,
         operation='SCALE',
         operation_state='FAILED_TEMP',
@@ -920,8 +1119,7 @@ def vnflcm_rollback_scale_in():
         operation_params='{"type": "SCALE_IN", "aspect_id": "SP1"}',
         error_point=7,
         id=constants.UUID,
-        created_at=datetime.datetime(2000, 1, 1, 1, 1, 1,
-                                     tzinfo=iso8601.UTC))
+        created_at=dt)
 
 
 def vnf_rollback():
@@ -939,15 +1137,15 @@ def vnf_dict():
                 'number\n  nfv:\n    type: json\nresources:\n  ' + \
                 'SP1_scale_out:\n    type: OS::Heat::ScalingPolicy\n' + \
                 '    properties:\n      auto_scaling_group_id: ' + \
-                '{get_resource: SP1_group}\n      adjustment_type: ' + \
+                '{get_resource: SP1}\n      adjustment_type: ' + \
                 'change_in_capacity\n      scaling_adjustment: 1\n  ' + \
-                'SP1_group:\n    type: OS::Heat::AutoScalingGroup\n    ' + \
+                'SP1:\n    type: OS::Heat::AutoScalingGroup\n    ' + \
                 'properties:\n      min_size: 0\n      desired_capacity:' + \
                 ' {get_param: current_num}\n      resource:\n        ' + \
                 'type: SP1_res.yaml\n        properties:\n          nfv:' + \
                 ' {get_param: nfv}\n      max_size: 3\n  SP1_scale_in:\n' + \
                 '    type: OS::Heat::ScalingPolicy\n    properties:\n' + \
-                '      auto_scaling_group_id: {get_resource: SP1_group}\n' + \
+                '      auto_scaling_group_id: {get_resource: SP1}\n' + \
                 '      adjustment_type: change_in_capacity\n      ' + \
                 'scaling_adjustment: -1\noutputs: {}\n'
     scale_g = '{\"scaleGroupDict\": { \"SP1\": { \"vdu\":' + \
@@ -958,7 +1156,7 @@ def vnf_dict():
            ' flavour for Sample VNF\n\nimports:\n' + \
            '  - etsi_nfv_sol001_common_types.yaml\n' + \
            '  - etsi_nfv_sol001_vnfd_types.yaml\n\n' + \
-           'topology_template:\n  node_templates:\n' + \
+           'topology_template:\n  groups:\n  node_templates:\n' + \
            '    VNF:\n      type: nec.ossmano.VNF\n' + \
            '      properties:\n' + \
            '        flavour_description: A simple flavour\n' + \
@@ -993,7 +1191,7 @@ def vnf_dict():
            '          size_of_storage: 1 GB\n' + \
            '          rdma_enabled: true\n        sw_image_data:\n' + \
            '          name: VirtualStorage\n' + \
-           '          version: \'0.4.0\'\n          checksum:\n' + \
+           '          version: \'0.5.2\'\n          checksum:\n' + \
            '            algorithm: sha-512\n' + \
            '            hash: 6513f21e44aa3da349f248188a44bc304a3' + \
            '653a04122d8fb4535423c8e1d14cd6a153f735bb0982e2161b5b5' + \
@@ -1082,9 +1280,129 @@ def vnf_dict():
             }
         }
     }
+
     return vnf_dict
 
 
+def vnf_dict_cnf():
+    vnf_dict = {
+        'attributes': {},
+        'status': 'ACTIVE',
+        'vnfd_id': 'e889e4fe-52fe-437d-b1e1-a690dc95e3f8',
+        'tenant_id': '13d2ca8de70d48b2a2e0dbac2c327c0b',
+        'vim_id': '3f41faa7-5630-47d2-9d4a-1216953c8887',
+        'instance_id': 'd1121d3c-368b-4ac2-b39d-835aa3e4ccd8',
+        'placement_attr': {'vim_name': 'kubernetes-vim'},
+        'id': '436aaa6e-2db6-4d6e-a3fc-e728b2f0ac56',
+        'name': 'cnf_create_1',
+        'vnfd': {
+            'attributes': {
+                'vnfd_simple': 'dummy'
+            }
+        }
+    }
+    return vnf_dict
+
+
+def vnfd_dict_cnf(vdu_num=1):
+    tacker_dir = os.getcwd()
+    def_dir = tacker_dir + "/samples/vnf_packages/Definitions/"
+    vnfd_dict = {
+        "tosca_definitions_version": "tosca_simple_yaml_1_2",
+        "description": "Sample VNF flavour for Sample VNF",
+        "imports": [
+            def_dir + "etsi_nfv_sol001_common_types.yaml",
+            def_dir + "etsi_nfv_sol001_vnfd_types.yaml",
+            def_dir + "helloworld3_types.yaml"],
+        "topology_template": {
+            "node_templates": {
+                "VNF": {
+                    "type": "company.provider.VNF",
+                    "properties": {
+                        "flavour_description": "A simple flavour"}}
+                # add VDU defininition later
+            },
+            "policies": [
+                {
+                    "scaling_aspects": {
+                        "type": "tosca.policies.nfv.ScalingAspects",
+                        "properties": {
+                            "aspects": {
+                                # add aspects later
+                            }}}},
+                # add following policies definitions later
+                # - tosca.policies.nfv.VduInitialDelta
+                # - tosca.policies.nfv.VduScalingAspectDeltas
+                # - tosca.policies.nfv.VduInstantiationLevels
+                {
+                    "instantiation_levels": {
+                        "type": "tosca.policies.nfv.InstantiationLevels",
+                        "properties": {
+                            "levels": {
+                                "instantiation_level_1": {
+                                    "description": "Smallest size",
+                                    "scale_info": {
+                                        # add scale_info later
+                                    }},
+                                "instantiation_level_2": {
+                                    "description": "Largest size",
+                                    "scale_info": {
+                                        # add scale_info later
+                                    }}
+                            },
+                            "default_level": "instantiation_level_1"}}}
+            ]
+        }
+    }
+    topology = vnfd_dict["topology_template"]
+    node_templates = topology["node_templates"]
+    policies = topology["policies"]
+    scaling_aspects = policies[0]["scaling_aspects"]
+    levels = policies[1]["instantiation_levels"]["properties"]["levels"]
+    for i in range(1, vdu_num + 1):
+        node_templates[f"VDU{i}"] = {
+            "type": "tosca.nodes.nfv.Vdu.Compute",
+            "properties": {
+                "name": f"vdu{i}",
+                "description": f"vdu{i} compute node",
+                "vdu_profile": {
+                    "min_number_of_instances": 1,
+                    "max_number_of_instances": 3}}}
+        scaling_aspects["properties"]["aspects"][f"vdu{i}_aspect"] = {
+            "name": f"vdu{i}_aspect",
+            "description": f"vdu{i} scaling aspect",
+            "max_scale_level": 2,
+            "step_deltas": ["delta_1"]}
+        policies.append({f"vdu{i}_initial_delta": {
+            "type": "tosca.policies.nfv.VduInitialDelta",
+            "properties": {
+                "initial_delta": {
+                    "number_of_instances": 0}},
+            "targets": [f"VDU{i}"]}})
+        policies.append({f"vdu{i}_scaling_aspect_deltas": {
+            "type": "tosca.policies.nfv.VduScalingAspectDeltas",
+            "properties": {
+                "aspect": f"vdu{i}_aspect",
+                "deltas": {
+                    "delta_1": {
+                        "number_of_instances": 1}}},
+            "targets": [f"VDU{i}"]}})
+        policies.append({f"vdu{i}_instantiation_levels": {
+            "type": "tosca.policies.nfv.VduInstantiationLevels",
+            "properties": {
+                "levels": {
+                    "instantiation_level_1": {
+                        "number_of_instances": 0},
+                    "instantiation_level_2": {
+                        "number_of_instances": 2}}},
+            "targets": [f"VDU{i}"]}})
+        levels["instantiation_level_1"]["scale_info"] = {
+            f"vdu{i}_aspect": {"scale_level": 0}}
+        levels["instantiation_level_2"]["scale_info"] = {
+            f"vdu{i}_aspect": {"scale_level": 2}}
+    return vnfd_dict
+
+
 class InjectContext(wsgi.Middleware):
     """Add a 'tacker.context' to WSGI environ."""
 
@@ -1112,27 +1430,41 @@ def wsgi_app_v1(fake_auth_context=None):
 VNFLCMOPOCC_RESPONSE = {
     '_links': {
         "self": {
-            "href": CONF.vnf_lcm.endpoint_url + '/vnflcm/v1/vnf_lcm_op_occs/'
+            "href": CONF.vnf_lcm.endpoint_url.rstrip("/") +
+            '/vnflcm/v1/vnf_lcm_op_occs/'
             'f26f181d-7891-4720-b022-b074ec1733ef'
         },
         "vnfInstance": {
-            "href": CONF.vnf_lcm.endpoint_url + '/vnflcm/v1/vnf_instances/'
+            "href": CONF.vnf_lcm.endpoint_url.rstrip("/") +
+            '/vnflcm/v1/vnf_instances/'
             'f26f181d-7891-4720-b022-b074ec1733ef'
         },
+        "retry": {
+            "href": CONF.vnf_lcm.endpoint_url.rstrip("/") +
+            '/vnflcm/v1/vnf_lcm_op_occs/'
+            'f26f181d-7891-4720-b022-b074ec1733ef/retry'
+        },
         "rollback": {
-            "href": CONF.vnf_lcm.endpoint_url + '/vnflcm/v1/vnf_lcm_op_occs/'
+            "href": CONF.vnf_lcm.endpoint_url.rstrip("/") +
+            '/vnflcm/v1/vnf_lcm_op_occs/'
             'f26f181d-7891-4720-b022-b074ec1733ef/rollback'
         },
         "grant": {
-            "href": CONF.vnf_lcm.endpoint_url + '/vnflcm/v1/vnf_lcm_op_occs/'
-            'f26f181d-7891-4720-b022-b074ec1733ef/grant'
-        }},
+            "href": CONF.vnf_lcm.endpoint_url.rstrip("/") +
+            '/vnflcm/v1/vnf_lcm_op_occs/'
+            'f26f181d-7891-4720-b022-b074ec1733ef/grant',
+        },
+        "fail": {
+            "href": CONF.vnf_lcm.endpoint_url.rstrip("/") +
+            '/vnflcm/v1/vnf_lcm_op_occs/'
+            'f26f181d-7891-4720-b022-b074ec1733ef/fail'}},
     'operationState': 'COMPLETED',
     'stateEnteredTime': datetime.datetime(1900, 1, 1, 1, 1, 1,
                             tzinfo=iso8601.UTC),
     'startTime': datetime.datetime(1900, 1, 1, 1, 1, 1,
                             tzinfo=iso8601.UTC),
     'vnfInstanceId': 'f26f181d-7891-4720-b022-b074ec1733ef',
+    'grantId': 'f26f181d-7891-4720-b022-b074ec1733ef',
     'operation': 'MODIFY_INFO',
     'isAutomaticInvocation': False,
     'operationParams': '{"is_reverse": False, "is_auto": False}',
@@ -1179,9 +1511,16 @@ VNFLCMOPOCC_RESPONSE = {
         }]
     },
     'changedInfo': {
-        'vimConnectionInfo': [],
-        'vimConnectionInfoDeleteIds': [],
-        'vnfPkgId': None,
+        'metadata': {'key': 'value'},
+        'vimConnectionInfo': [
+            {"id": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+             "vimId": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+             "vimType": 'openstack',
+             'interfaceInfo': {},
+             "accessInfo": {"key1": 'value1', "key2": 'value2'},
+             "extra": {}}],
+        'vimConnectionInfoDeleteIds': ['f8c35bd0-4d67-4436-9f11-14b8a84c92bb'],
+        'vnfPkgId': 'f26f181d-7891-4720-b022-b074ec1733ef',
         'vnfInstanceName': 'fake_name',
         'vnfInstanceDescription': "fake_vnf_instance_description",
         'vnfdId': 'f26f181d-7891-4720-b022-b074ec1733ef',
@@ -1189,7 +1528,24 @@ VNFLCMOPOCC_RESPONSE = {
         'vnfProductName': 'fake_vnf_product_name',
         'vnfSoftwareVersion': 'fake_vnf_software_version',
         'vnfdVersion': 'fake_vnfd_version'
-    }
+    },
+    "changedExtConnectivity": [{
+        "id": constants.UUID,
+        "resourceHandle": {
+            "vimConnectionId": constants.UUID,
+            "resourceId": constants.UUID,
+            "vimLevelResourceType": "OS::Neutron::Net",
+        },
+        "extLinkPorts": [{
+            "id": constants.UUID,
+            "resourceHandle": {
+                "vimConnectionId": constants.UUID,
+                "resourceId": constants.UUID,
+                "vimLevelResourceType": "OS::Neutron::Port",
+            },
+            "cpInstanceId": constants.UUID,
+        }]
+    }]
 }
 
 VNFLCMOPOCC_INDEX_RESPONSE = [VNFLCMOPOCC_RESPONSE]
@@ -1276,11 +1632,20 @@ def fake_vnf_lcm_op_occs():
     }
     resource_changes_obj = objects.ResourceChanges(**resource_changes)
 
+    vim_connection_info = {
+        "id": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+        "vim_id": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+        "vim_type": 'openstack',
+        "access_info": {"key1": 'value1', "key2": 'value2'}}
     changed_info = {
         "vnf_instance_name": "fake_name",
         "vnf_instance_description":
             "fake_vnf_instance_description",
-        "metadata": {},
+        'metadata': {'key': 'value'},
+        'vim_connection_info': [VimConnectionInfo(**vim_connection_info)],
+        'vim_connection_info_delete_ids': [
+            'f8c35bd0-4d67-4436-9f11-14b8a84c92bb'],
+        'vnf_pkg_id': 'f26f181d-7891-4720-b022-b074ec1733ef',
         "vnfd_id": "f26f181d-7891-4720-b022-b074ec1733ef",
         "vnf_provider": "fake_vnf_provider",
         "vnf_product_name": "fake_vnf_product_name",
@@ -1289,28 +1654,245 @@ def fake_vnf_lcm_op_occs():
     }
     changed_info_obj = objects.VnfInfoModifications(**changed_info)
 
+    changed_ext_connectivity = [{
+        "id": constants.UUID,
+        "resource_handle": {
+            "vim_connection_id": constants.UUID,
+            "resource_id": constants.UUID,
+            "vim_level_resource_type": "OS::Neutron::Net",
+        },
+        "ext_link_ports": [{
+            "id": constants.UUID,
+            "resource_handle": {
+                "vim_connection_id": constants.UUID,
+                "resource_id": constants.UUID,
+                "vim_level_resource_type": "OS::Neutron::Port",
+            },
+            "cp_instance_id": constants.UUID,
+        }]
+    }]
+    changed_ext_connectivity_obj = \
+        [objects.ExtVirtualLinkInfo.obj_from_primitive(
+         chg_ext_conn, context) for chg_ext_conn in
+         changed_ext_connectivity]
+
+    dt = datetime.datetime(1900, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
     vnf_lcm_op_occs = {
         'id': constants.UUID,
         'operation_state': 'COMPLETED',
-        'state_entered_time': datetime.datetime(1900, 1, 1, 1, 1, 1,
-                                tzinfo=iso8601.UTC),
-        'start_time': datetime.datetime(1900, 1, 1, 1, 1, 1,
-                                tzinfo=iso8601.UTC),
+        'state_entered_time': dt,
+        'start_time': dt,
         'vnf_instance_id': constants.UUID,
+        'grant_id': constants.UUID,
         'operation': 'MODIFY_INFO',
         'is_automatic_invocation': False,
         'operation_params': '{"is_reverse": False, "is_auto": False}',
         'is_cancel_pending': False,
         'error': error_obj,
         'resource_changes': resource_changes_obj,
-        'changed_info': changed_info_obj
+        'changed_info': changed_info_obj,
+        'changed_ext_connectivity': changed_ext_connectivity_obj,
     }
 
     return vnf_lcm_op_occs
 
 
-def return_vnf_lcm_opoccs_obj():
+def return_vnf_lcm_opoccs_obj(**updates):
     vnf_lcm_op_occs = fake_vnf_lcm_op_occs()
+
+    if updates:
+        vnf_lcm_op_occs.update(**updates)
+
     obj = objects.VnfLcmOpOcc(**vnf_lcm_op_occs)
 
     return obj
+
+
+def vnflcm_op_occs_retry_data(error_point=7, operation='INSTANTIATE',
+        operation_state='FAILED_TEMP'):
+    now = datetime.datetime(2000, 1, 1, 1, 1, 1, tzinfo=iso8601.UTC)
+    return objects.VnfLcmOpOcc(
+        state_entered_time=now,
+        start_time=now,
+        vnf_instance_id=uuidsentinel.vnf_instance_id,
+        operation=operation,
+        operation_state=operation_state,
+        is_automatic_invocation=False,
+        operation_params='{}',
+        error_point=error_point,
+        id=constants.UUID,
+        created_at=now)
+
+
+def vnf_data(status='ACTIVE'):
+    return tacker.db.vnfm.vnfm_db.VNF(id=constants.UUID,
+        vnfd_id=uuidsentinel.vnfd_id,
+        name='test',
+        status=status,
+        vim_id=uuidsentinel.vim_id)
+
+
+def return_vnf_lcm_opoccs_list():
+    vnf_lcm_op_occs = fake_vnf_lcm_op_occs()
+    obj = objects.VnfLcmOpOcc(**vnf_lcm_op_occs)
+
+    return [obj]
+
+
+def get_change_ext_conn_request_body():
+    change_ext_conn_req_body = {
+        "extVirtualLinks": [{
+            "id": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+            "vimConnectionId": '2b3beeff-d4a1-4dc7-a1f8-066f92cfcb75',
+            "resourceId": 'e08f5e67-55de-4c4a-815b-cf3f1e2bae04',
+            "extCps": [{
+                "cpdId": 'VDU2_CP2',
+                "cpConfig": [{
+                    "cpInstanceId": '924d0ea7-786d-468b-bf45-65bfd483ee79',
+                    "linkPortId": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+                    "cpProtocolData": [{
+                        "layerProtocol": 'IP_OVER_ETHERNET',
+                        "ipOverEthernet": {
+                            "macAddress":
+                                'fa:16:3e:11:11:11',
+                            "ipAddresses": [{
+                                "type": "IPV4",
+                                "fixedAddresses": ["22.22.1.20"],
+                                "subnetId":
+                                    '497b7a75-6c10-4a74-85fa-83d498da2501'
+                            }]
+                        }
+                    }]
+                }]
+            }],
+            "extLinkPorts": [{
+                "id": 'decd78d2-993c-4112-9a8f-1ad54cade4d7',
+                "resourceHandle": {
+                    "resourceId": 'cb602960-05ee-4e03-8fe2-ea0b64e08332',
+                    "vimConnectionId": '2b3beeff-d4a1-4dc7-a1f8-066f92cfcb75',
+                    "vimLevelResourceType":
+                        'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+                }
+            }],
+            "vimConnectionInfo": [{
+                "id": '2b3beeff-d4a1-4dc7-a1f8-066f92cfcb75',
+                "vimId": 'f8c35bd0-4d67-4436-9f11-14b8a84c92aa',
+                "vimType": 'openstack',
+                "interfaceInfo": {"key1": 'value1', "key2": 'value2'},
+                "accessInfo": {"key1": 'value1', "key2": 'value2'}
+            }],
+        }]
+    }
+
+    return change_ext_conn_req_body
+
+
+def get_change_ext_conn_request_obj():
+    """Return ChangeExtConnRequest Object
+
+    obj_from_primitive() needs snake_case dictionary
+    """
+    body = utils.convert_camelcase_to_snakecase(
+        get_change_ext_conn_request_body())
+    return ChangeExtConnRequest.obj_from_primitive(
+        body, context)
+
+
+def _fake_subscription_obj(**updates):
+    subscription = {
+        'id': uuidsentinel.subscription_id,
+        'filter': {
+            "vnfInstanceSubscriptionFilter": {
+                "vnfdIds": [uuidsentinel.vnfd_id],
+                "vnfProductsFromProviders": {
+                    "vnfProvider": "Vnf Provider 1",
+                    "vnfProducts": [
+                        {
+                            "vnfProductName": "Vnf Product 1",
+                            "versions": [
+                                {
+                                    "vnfSoftwareVersion": "v1",
+                                    "vnfdVersions": [
+                                        "vnfd.v1.1"
+                                    ]
+                                }
+                            ]
+                        }
+                    ]
+                },
+                "vnfInstanceIds": [
+                    uuidsentinel.vnf_instance_id
+                ],
+                "vnfInstanceNames": ["Vnf Name 1"]
+            },
+            "notificationTypes": [
+                "VnfLcmOperationOccurrenceNotification",
+                "VnfIdentifierCreationNotification",
+                "VnfIdentifierDeletionNotification"
+            ],
+            "operationTypes": [
+                "INSTANTIATE",
+                "SCALE",
+                "TERMINATE",
+                "HEAL",
+                "CHANGE_EXT_CONN",
+                "MODIFY_INFO"
+            ],
+            "operationStates": [
+                "STARTING",
+                "PROCESSING",
+                "COMPLETED",
+                "FAILED_TEMP",
+                "FAILED",
+                "ROLLING_BACK",
+                "ROLLED_BACK"
+            ]
+        },
+        'callback_uri': 'http://localhost/sample_callback_uri'}
+
+    if updates:
+        subscription.update(**updates)
+
+    return subscription
+
+
+def return_subscription_object(**updates):
+    vnf_lcm_subscription = _fake_subscription_obj(**updates)
+    return vnf_lcm_subscription
+
+
+def return_vnf_subscription_list(**updates):
+    vnc_lcm_subscription = return_subscription_object(**updates)
+    return [vnc_lcm_subscription]
+
+
+def _subscription_links(subscription_dict):
+    links = {
+        "_links": {
+            "self": {
+                "href":
+                "{endpoint}/vnflcm/v1/subscriptions/{id}".format(
+                    id=subscription_dict['id'],
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"))
+            }
+        }
+    }
+    subscription_dict.update(links)
+
+    return subscription_dict
+
+
+def return_subscription_obj(**updates):
+    subscription = _fake_subscription_obj(**updates)
+    subscription['filter'] = json.dumps(subscription['filter'])
+    obj = objects.LccnSubscriptionRequest(**subscription)
+
+    return obj
+
+
+def fake_subscription_response(**updates):
+    data = _fake_subscription_obj(**updates)
+    data = utils.convert_snakecase_to_camelcase(data)
+    data = _subscription_links(data)
+
+    return data
