@@ -19,6 +19,7 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from toscaparser.utils import yamlparser
 
+from tacker.common import exceptions
 from tacker.common import log
 from tacker.extensions import common_services as cs
 from tacker.extensions import vnfm
@@ -32,10 +33,10 @@ CONF = cfg.CONF
 class TOSCAToKubernetes(object):
 
     def __init__(self, vnf, core_v1_api_client,
-                 extension_api_client, scaling_api_client):
+                 app_v1_api_client, scaling_api_client):
         self.vnf = vnf
         self.core_v1_api_client = core_v1_api_client
-        self.extension_api_client = extension_api_client
+        self.app_v1_api_client = app_v1_api_client
         self.scaling_api_client = scaling_api_client
         self.attributes = {}
         self.vnfd_yaml = None
@@ -56,8 +57,10 @@ class TOSCAToKubernetes(object):
         tosca_kube_objects = self.generate_tosca_kube_objects()
         transformer = translate_outputs.Transformer(
             core_v1_api_client=self.core_v1_api_client,
-            extension_api_client=self.extension_api_client,
-            scaling_api_client=self.scaling_api_client)
+            app_v1_api_client=self.app_v1_api_client,
+            scaling_api_client=self.scaling_api_client,
+            k8s_client_dict=None
+        )
         kubernetes_objects = transformer.transform(tosca_kube_objects)
         deployment_names = transformer.deploy(
             kubernetes_objects=kubernetes_objects)
@@ -70,8 +73,8 @@ class TOSCAToKubernetes(object):
         self.attributes = self.vnf['vnfd']['attributes'].copy()
         self.vnfd_yaml = self.attributes.pop('vnfd', None)
         if self.vnfd_yaml is None:
-            LOG.info("VNFD is not provided, so no vnf is created !!")
-            return
+            LOG.error("VNFD is not provided, so no vnf is created !!")
+            raise exceptions.InvalidInput("VNFD template is None.")
         LOG.debug('vnfd_yaml %s', self.vnfd_yaml)
         vnfd_dict = yamlparser.simple_ordered_parse(self.vnfd_yaml)
         LOG.debug('vnfd_dict %s', vnfd_dict)
@@ -92,7 +95,7 @@ class TOSCAToKubernetes(object):
                     if value['get_input'] in paramvalues:
                         original[key] = paramvalues[value['get_input']]
                     else:
-                        LOG.debug('Key missing Value: %s', key)
+                        LOG.error('Key missing Value: %s', key)
                         raise cs.InputValuesMissing(key=key)
                 else:
                     self._update_params(value, paramvalues)
@@ -109,7 +112,7 @@ class TOSCAToKubernetes(object):
                     if 'get_input' in str(node):
                         self._update_params(node, param_vattrs_dict)
             except Exception as e:
-                LOG.debug("Not Well Formed: %s", str(e))
+                LOG.error("Not Well Formed: %s", str(e))
                 raise vnfm.ParamYAMLNotWellFormed(
                     error_msg_details=str(e))
             else:
