@@ -13,12 +13,13 @@
 #
 
 import netaddr
+from urllib import error as urlerr
+from urllib import request as urlreq
 
 from oslo_config import cfg
 from oslo_log import log as logging
-import six.moves.urllib.error as urlerr
-import six.moves.urllib.request as urlreq
 
+from tacker._i18n import _
 from tacker.common import log
 from tacker.vnfm.monitor_drivers import abstract_driver
 
@@ -74,7 +75,7 @@ class VNFMonitorHTTPPing(abstract_driver.VNFMonitorAbstractDriver):
                 urlreq.urlopen(url, timeout=timeout)
                 return True
             except urlerr.URLError:
-                LOG.warning('Unable to reach to the url %s', url)
+                LOG.error('Unable to reach to the url %s', url)
         return 'failure'
 
     @log.log
